package emulator.hardware.debug;

import emulator.hardware.CPU;
import emulator.hardware.CPUExecutionObserver;

public class CpuProfiler implements CPUExecutionObserver
{
	private AddressStatistics[] statistics;
	private int current_address = -1;
	private long last_cycle_count = -1;
	
	public CpuProfiler(int address_range)
	{
		statistics = new AddressStatistics[address_range];
		for (int i = 0; i < address_range; i++)
			statistics[i] = new AddressStatistics(i);
	}
	
	public void reset()
	{
		for (AddressStatistics s : statistics)
			s.reset();
	}
	
	@Override
	public void notifyPreExecute(CPU cpu) {
		current_address = cpu.getPC();
		last_cycle_count = cpu.getTicks();
	}

	@Override
	public void notifyPostExecute(CPU cpu) {
		assert(current_address >= 0);
		assert(last_cycle_count >= 0);
		statistics[current_address].countExecution((int) (cpu.getTicks() - last_cycle_count));
		current_address = -1;
		last_cycle_count = -1;
	}
	
	public AddressStatistics getStatisticsForAddress(int address)
	{
		return statistics[address];
	}
}
