package emulator.hardware.nmos6502.commands;

import emulator.EmulatorException;
import emulator.hardware.nmos6502.Command;
import emulator.hardware.nmos6502.CommandSet;
import emulator.hardware.nmos6502.Cpu6502;
import emulator.hardware.nmos6502.Flags6502;
import emulator.hardware.nmos6502.Operand;
import emulator.hardware.nmos6502.operands.Displacement;

/** BCS (Branch on Carry Set) */
public class BCS implements Command {

	@Override
	public void execute(Cpu6502 cpu, Operand operand) throws EmulatorException {
		if (cpu.getFlags().getCarry())
			cpu.branch(operand.getByte());
	}

	@Override
	public String getName() {
		return "BCS";
	}

	@Override
	public void register(CommandSet cset) {
		cset.defineCommand(0xB0, this, new Displacement(Flags6502.BIT_CARRY, true));
	}

}
