package emulator.hardware.nmos6502.commands;

import emulator.EmulatorException;
import emulator.hardware.HwByte;
import emulator.hardware.nmos6502.Command;
import emulator.hardware.nmos6502.CommandSet;
import emulator.hardware.nmos6502.Cpu6502;
import emulator.hardware.nmos6502.Flags6502;
import emulator.hardware.nmos6502.Operand;
import emulator.hardware.nmos6502.operands.*;

/** DEC (DECrement memory)
 *  
 *  Affects Flags: S Z
 *  
 *  MODE           SYNTAX       HEX LEN TIM
 *  Zero Page     DEC $44       $C6  2   5
 *  Zero Page,X   DEC $44,X     $D6  2   6
 *  Absolute      DEC $4400     $CE  3   6
 *  Absolute,X    DEC $4400,X   $DE  3   7
 */
public class DEC implements Command {

	@Override
	public void execute(Cpu6502 cpu, Operand operand) throws EmulatorException {
		HwByte value = operand.getByte();
		value.inc(-1);
		operand.setByte(value);
		Flags6502 flags = cpu.getFlags();
		flags.setNZFromValue(value);
		cpu.setFlags(flags);
	}

	@Override
	public String getName() {
		return "DEC";
	}

	@Override
	public void register(CommandSet cset) {
		cset.defineCommand(0xC6, this, new ZeroPage(), 4);
		cset.defineCommand(0xD6, this, new ZeroPageX(), 4);
		cset.defineCommand(0xCE, this, new Absolute(), 4);
		cset.defineCommand(0xDE, this, new AbsoluteX(false), 5);
	}

}
