package emulator.hardware.nmos6502.commands;

import emulator.hardware.HwByte;
import emulator.hardware.memory.UnmappedMemoryException;
import emulator.hardware.nmos6502.Command;
import emulator.hardware.nmos6502.CommandSet;
import emulator.hardware.nmos6502.Cpu6502;
import emulator.hardware.nmos6502.Flags6502;
import emulator.hardware.nmos6502.Operand;
import emulator.hardware.nmos6502.operands.NoOperand;
import emulator.hardware.nmos6502.operands.OperandException;

/** TAY (Transfer A to Y) */
public class TAY implements Command {

	@Override
	public void execute(Cpu6502 cpu, Operand operand) throws OperandException,
			UnmappedMemoryException, IllegalOpcodeException {
		HwByte value = cpu.getAccu();
		cpu.setY(value);
		Flags6502 flags = cpu.getFlags();
		flags.setNZFromValue(value);
		cpu.setFlags(flags);
	}

	@Override
	public String getName() {
		return "TAY";
	}

	@Override
	public void register(CommandSet cset) {
		cset.defineCommand(0xA8, this, new NoOperand());
	}

}
