package emulator.hardware.nmos6502.operands;

import emulator.assembler.SymbolTable;
import emulator.hardware.HwWord;
import emulator.hardware.debug.BusWatchException;
import emulator.hardware.memory.UnmappedMemoryException;
import emulator.hardware.nmos6502.Cpu6502;
import emulator.support.AbsoluteXOperandEval;
import emulator.support.OperandEval;
import emulator.util.MemoryPtr;

public class AbsoluteX extends ByteOperand {

	private boolean add_cycles_on_crossing = true;
	private HwWord abs_addr = new HwWord();

	public AbsoluteX() { super(); }
	public AbsoluteX(boolean add_cycles_on_crossing)
	{
		super();
		this.add_cycles_on_crossing = add_cycles_on_crossing;
	}
	
	@Override
	public void init(Cpu6502 cpu) throws UnmappedMemoryException, BusWatchException {
		super.init(cpu);
		HwWord addr = new HwWord(cpu.fetchProgramWord());
		abs_addr.setNumber(addr.getNumber());
		addr.add(cpu.getX());
		setAddress(addr);
	}

	@Override
	public String getString() {
		return "$"+abs_addr+",X";
	}

	@Override
	public int getByteCount() {
		return 2;
	}

	@Override
	public String getString(MemoryPtr mem_ref, SymbolTable symbols) {
		return symbols.getLabelOrAddress(mem_ref.getWordOp())+",X";
	}

	@Override
	public int getExtraCycles() {
		if (add_cycles_on_crossing &&
				!abs_addr.hi().equals(getAddress().hi()))
			return 3;
		return 2;
	}

	@Override
	public OperandEval getEvaluator(MemoryPtr mem_ref) {
		return new AbsoluteXOperandEval(mem_ref);
	}
}