package emulator.hardware.video.test;

import emulator.EmulatorException;
import emulator.GUI.display.VicDisplay;
import emulator.GUI.display.VicScreen;
import emulator.hardware.bus.VideoBus;
import emulator.hardware.memory.MemoryBlock;
import emulator.hardware.video.Vic6561;

public class TestVic {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		NullClock clock = new NullClock();
		CountingClock counting_clock = new CountingClock();
		
		MemoryBlock memory = new MemoryBlock(65536);
		Vic6561 vic = new Vic6561();
		
		VicDisplay null_display = new NullDisplay();
		VicDisplay display_1x1 = new VicScreen(1, VicDisplay.MAX_SCREEN_WIDTH, 1, VicDisplay.MAX_SCREEN_HEIGHT);
		VicDisplay display_3x2 = new VicScreen(3, 3*VicDisplay.MAX_SCREEN_WIDTH, 2, 2*VicDisplay.MAX_SCREEN_HEIGHT);

		vic.attach(new VideoBus(memory), counting_clock);
		
		System.out.println("Counting cycles per frame");
		try {
			vic.refreshDisplay(null_display);
			System.out.println("Counted ticks: "+counting_clock.getCounterValue() + "  " +
							   "Ticks for resolution: 284x312/4 =" + 284*312/4);
		} catch (EmulatorException e1) {
			e1.printStackTrace();
		}
		
		vic.attach(new VideoBus(memory), clock);
		
		int cycles = 10;
		
		System.out.println("Performance test without display");
		try {
			long now = System.nanoTime();
			for (int i = 0; i < cycles; i++)
				vic.refreshDisplay(null_display);
			long diff = System.nanoTime() - now;
			double time_ms = (double)diff/1e6/cycles;
			System.out.println("elapsed time: " + time_ms + " ms  " + (int)(1e3/time_ms) + " Hz");
		} catch (EmulatorException e) {
			e.printStackTrace();
		}
		
		System.out.println("Performance test with 1x1 display");
		try {
			long now = System.nanoTime();
			for (int i = 0; i < cycles; i++)
				vic.refreshDisplay(display_1x1);
			long diff = System.nanoTime() - now;
			double time_ms = (double)diff/1e6/cycles;
			System.out.println("elapsed time: " + time_ms + " ms  " + (int)(1e3/time_ms) + " Hz");
		} catch (EmulatorException e) {
			e.printStackTrace();
		}
		
		System.out.println("Performance test with 3x2 display");
		try {
			long now = System.nanoTime();
			for (int i = 0; i < cycles; i++)
				vic.refreshDisplay(display_3x2);
			long diff = System.nanoTime() - now;
			double time_ms = (double)diff/1e6/cycles;
			System.out.println("elapsed time: " + time_ms + " ms  " + (int)(1e3/time_ms) + " Hz");
		} catch (EmulatorException e) {
			e.printStackTrace();
		}
	}

}
