package emulator.unittest.hardware.debug;

import static org.junit.Assert.assertEquals;

import org.junit.Before;
import org.junit.Test;

import emulator.EmulatorException;
import emulator.hardware.HwWord;
import emulator.hardware.debug.BusWatchException;
import emulator.hardware.debug.WatchWrites;
import emulator.hardware.memory.SparseMemory;

public class WatchWritesTest {

	private SparseMemory memory;
	private WatchWrites watcher;

	@Before
	public void setUp() throws Exception {
		memory = new SparseMemory();
		watcher = new WatchWrites(memory, new HwWord(0xC000), new HwWord(0xFFFF));
	}

	@Test(expected= BusWatchException.class)
	public void testWriteWatch1() throws EmulatorException {
		watcher.write(0xC000, 0);
	}

	@Test(expected= BusWatchException.class)
	public void testWriteWatch2() throws EmulatorException {
		watcher.write(0xC001, 0);
	}

	@Test(expected= BusWatchException.class)
	public void testWriteWatch3() throws EmulatorException {
		watcher.write(0xFFFF, 0);
	}

	@Test
	public void testWriteThroughBorder() throws EmulatorException {
		watcher.write(0xBFFF, 0xF7);
		assertEquals(0xF7, memory.read(0xBFFF));
	}
	
	@Test
	public void testWriteThrough() throws EmulatorException {
		watcher.write(0xA000, 0xF7);
		assertEquals(0xF7, memory.read(0xA000));
	}
	
}
