package emulator.unittest.hardware.nmos6502.commands;

import static org.junit.Assert.*;

import org.junit.Test;

import emulator.EmulatorException;
import emulator.hardware.HwByte;
import emulator.hardware.nmos6502.Cpu6502;
import emulator.hardware.nmos6502.Flags6502;
import emulator.hardware.nmos6502.commands.LSR;
import emulator.unittest.hardware.nmos6502.MockOperand;

public class LSRTest {

	@Test
	public void testRotatePositive() throws EmulatorException {
		executeAndCheckLSR(0, 0x80, 0, 0x40);
	}

	@Test
	public void testRotateNegative() throws EmulatorException {
		executeAndCheckLSR(Flags6502.FLAG_CARRY, 0, Flags6502.FLAG_ZERO, 0);
	}

	@Test
	public void testRotateZero() throws EmulatorException {
		executeAndCheckLSR(0, 0x01, Flags6502.FLAG_ZERO|Flags6502.FLAG_CARRY, 0);
	}

	private void executeAndCheckLSR(int flags, int val, int expected_flags, int expected_val)
			throws EmulatorException {
		Cpu6502 cpu = new Cpu6502();
		MockOperand operand = new MockOperand(val);
		cpu.setFlags(new Flags6502(flags));
		new LSR().execute(cpu, operand);
		assertEquals(new HwByte(expected_val), operand.getByte());
		assertEquals(new Flags6502(expected_flags), cpu.getFlags());
	}
}
