package emulator.unittest.hardware.nmos6502.commands;


import static org.junit.Assert.assertEquals;

import org.junit.Before;
import org.junit.Test;

import emulator.EmulatorException;
import emulator.hardware.HwByte;
import emulator.hardware.nmos6502.Cpu6502;
import emulator.hardware.nmos6502.Flags6502;
import emulator.hardware.nmos6502.commands.TXA;
import emulator.hardware.nmos6502.operands.NoOperand;

public class TXATest {

	private Cpu6502 cpu;
	private TXA txa;

	@Before
	public void setUp() throws Exception {
		cpu = new Cpu6502();
		txa = new TXA();
	}

	@Test
	public void testTransferZero() throws EmulatorException {
		executeAndcheckTXA(0, Flags6502.FLAG_ZERO);
	}

	@Test
	public void testTransferPositive() throws EmulatorException {
		executeAndcheckTXA(5, 0);
	}

	@Test
	public void testTransferNegative() throws EmulatorException {
		executeAndcheckTXA(0x81, Flags6502.FLAG_NEGATIVE);
	}

	private void executeAndcheckTXA(int target, int expected_flags)	throws EmulatorException {
		cpu.setX(new HwByte(target));
		cpu.setAccu(new HwByte(0xFF));
		txa.execute(cpu, new NoOperand());
		assertEquals(target, cpu.getX().getNumber());
		assertEquals(target, cpu.getAccu().getNumber());
		assertEquals(new Flags6502(expected_flags), cpu.getFlags());
	}

}
