package emulator.unittest.hardware.nmos6502.timing;


import static org.junit.Assert.*;

import org.junit.Test;

import emulator.EmulatorException;

public class IndirectYTest {

	@Test
	public void testADC() throws EmulatorException
	{
		int opcode = 0x71;
		assertEquals(5, getCycles(opcode, false));
		assertEquals(6, getCycles(opcode, true));
	}
	
	@Test
	public void testAND() throws EmulatorException
	{
		int opcode = 0x31;
		assertEquals(5, getCycles(opcode, false));
		assertEquals(6, getCycles(opcode, true));
	}
	
	@Test
	public void testCMP() throws EmulatorException
	{
		int opcode = 0xD1;
		assertEquals(5, getCycles(opcode, false));
		assertEquals(6, getCycles(opcode, true));
	}
	
	@Test
	public void testEOR() throws EmulatorException
	{
		int opcode = 0x51;
		assertEquals(5, getCycles(opcode, false));
		assertEquals(6, getCycles(opcode, true));
	}
	
	@Test
	public void testLDA() throws EmulatorException
	{
		int opcode = 0xB1;
		assertEquals(5, getCycles(opcode, false));
		assertEquals(6, getCycles(opcode, true));
	}
	
	@Test
	public void testORA() throws EmulatorException
	{
		int opcode = 0x11;
		assertEquals(5, getCycles(opcode, false));
		assertEquals(6, getCycles(opcode, true));
	}
	
	@Test
	public void testSBC() throws EmulatorException
	{
		int opcode = 0xF1;
		assertEquals(5, getCycles(opcode, false));
		assertEquals(6, getCycles(opcode, true));
	}
	
	@Test
	public void testSTA() throws EmulatorException
	{
		int opcode = 0x91;
		assertEquals(5, getCycles(opcode, false));
		assertEquals(6, getCycles(opcode, true));
	}
	
	private int getCycles(int opcode, boolean cross_page) throws EmulatorException
	{
		TimingTestBase test_base = new TimingTestBase();
		test_base.setMemoryWord(0x44, 0x87);
		test_base.setRegisters(0, 0, (cross_page ? 0xC0 : 0), 0);
		test_base.setCommand(0, opcode, 0x44, 0);
		test_base.execute();
		return test_base.getElapsedCycles();
	}
}
